@@ -3,16 +3,17 @@ import sys
 import copy
 import zlib
 import pickle
+import json
 import shutil
 from importlib import import_module
 from itertools import islice
 
-from .utils_novel import get_cb_settings, request_to_dict
+from .utils_novel import get_cb_settings, request_to_dict, validate_results
 
 import six
 from scrapy import signals
 from scrapy.crawler import Crawler
-from scrapy.exceptions import NotConfigured
+from scrapy.exceptions import NotConfigured, _InvalidOutput
 from scrapy.http import Request, Response
 from scrapy.item import Item
 from scrapy.utils.conf import (build_component_list, closest_scrapy_cfg,
@@ -48,15 +49,15 @@ def create_instance(objcls, settings, crawler, *args, **kwargs):
 
 
 def get_project_dirs():
+    outer_dir = inner_dir = ""
     closest_cfg = closest_scrapy_cfg()
     if closest_cfg:
         outer_dir = os.path.dirname(closest_cfg)
-    inner_dir = ""
     if os.environ.get('SCRAPY_PROJECT'):
         inner_dir = os.environ.get('SCRAPY_PROJECT')
     if outer_dir and inner_dir:
         return (outer_dir, inner_dir)
-    
+
     init_env()
     scrapy_module = os.environ.get('SCRAPY_SETTINGS_MODULE')
     if scrapy_module is None and not outer_dir:
@@ -73,6 +74,7 @@ def get_project_dirs():
     except ImportError:
         raise Exception("Project configuration awry")
 
+
 def get_middlewares(spider):
     full_list = build_component_list(
         spider.settings.getwithbase('SPIDER_MIDDLEWARES'))
@@ -153,8 +155,7 @@ def response_to_dict(response):
     return {
         'cls': '{}.{}'.format(
             type(response).__module__,
-            getattr(type(response), '__qualname__', None) or
-            getattr(type(response), '__name__', None)
+            getattr(type(response), '__qualname__', None) or getattr(type(response), '__name__', None)
         ),
         'url': response.url,
         'status': response.status,
@@ -197,7 +198,7 @@ def parse_request(request, spider, cb_settings):
     if not _request['callback']:
         _request['callback'] = 'parse'
 
-    clean_headers(_request['headers'], spider.settings, cb_settings)
+    _clean_headers(_request['headers'], spider.settings, cb_settings)
 
     _meta = {}
     for key, value in _request.get('meta').items():
@@ -208,17 +209,66 @@ def parse_request(request, spider, cb_settings):
     return _request
 
 
-def clean_request(request, spider_settings, cb_settings):
-    skipped_global = spider_settings.get('TESTMASTER_REQUEST_SKIPPED_FIELDS', default=[])
+def _decode_dict(data):
+    decoded = {}
+    for key, value in data.items():
+        if isinstance(key, bytes):
+            key = key.decode()
+        if isinstance(value, bytes):
+            value = value.decode()
+        if isinstance(value, list) and len(value) > 0:
+            if isinstance(value[0], bytes):
+                value = value[0].decode()
+        decoded[key] = value
+    return decoded
+
+
+def _clean_splash(meta, spider_settings, cb_settings):
+    splash_headers = meta.get('splash', {}).get('splash_headers', {})
+    excluded_global = spider_settings.get(
+        'TESTMASTER_EXCLUDED_HEADERS', default=[])
     try:
-        skipped_local = cb_settings.REQUEST_SKIPPED_FIELDS
+        excluded_local = cb_settings.EXCLUDED_HEADERS
     except AttributeError:
-        skipped_local = []
-    skipped_fields = skipped_local if skipped_local else skipped_global
-    _clean(request, skipped_fields)
+        excluded_local = []
+    excluded = excluded_local if excluded_local else excluded_global
+
+    included_global = spider_settings.get(
+        'TESTMASTER_INCLUDED_AUTH_HEADERS', default=[])
+    try:
+        included_local = cb_settings.INCLUDED_AUTH_HEADERS
+    except AttributeError:
+        included_local = []
+    included = included_local if included_local else included_global
+    if 'Authorization' not in included or 'Authorization' in excluded:
+        splash_headers.pop('Authorization', None)
+    # deliberate inclusion!
+    if 'Authorization' in splash_headers:
+        try:
+            splash_headers['Authorization'] = splash_headers['Authorization'].decode()
+        except AttributeError:
+            pass
 
 
-def clean_headers(headers, spider_settings, cb_settings, mode=""):
+def _process_for_json(data):
+    def _is_jsonable(short_dict):
+        try:
+            json.dumps(short_dict)
+            return True
+        except:
+            return False
+    to_delete = []
+    for k, v in data.items():
+        if not _is_jsonable({k: v}):
+            try:
+                data[k] = str(v)
+            except:
+                to_delete.append(k)
+    for d in to_delete:
+        del data[d]
+
+
+def _clean_headers(headers, spider_settings, cb_settings, mode=""):
     excluded_global = spider_settings.get('TESTMASTER_EXCLUDED_HEADERS', default=[])
     try:
         excluded_local = cb_settings.EXCLUDED_HEADERS
@@ -239,16 +289,30 @@ def clean_headers(headers, spider_settings, cb_settings, mode=""):
         headers.pop(header, None)
         headers.pop(header.encode(), None)
     if mode == "decode":
-        decoded = {}
-        for key,value in headers.items():
-            decoded[key.decode()] = value[0].decode() 
-        return decoded
+        headers = _decode_dict(headers)
+    return headers
+
+
+def clean_request(request, spider_settings, cb_settings):
+    skipped_global = spider_settings.get('TESTMASTER_REQUEST_SKIPPED_FIELDS', default=[])
+    try:
+        skipped_local = cb_settings.REQUEST_SKIPPED_FIELDS
+    except AttributeError:
+        skipped_local = []
+    skipped_fields = skipped_local if skipped_local else skipped_global
+    _clean(request, skipped_fields)
+    request = _decode_dict(request)
+    request['headers'] = _clean_headers(request['headers'], spider_settings,
+                                        cb_settings, mode="decode")
+    _clean_splash(request['meta'], spider_settings, cb_settings)
+    _process_for_json(request['meta'])
+    return request
 
 
 def clean_item(item, spider_settings, cb_settings):
     skipped_global = spider_settings.get('TESTMASTER_SKIPPED_FIELDS', default=[])
     try:
-        skipped_local = cb_settings.REQUEST_SKIPPED_FIELDS
+        skipped_local = cb_settings.SKIPPED_FIELDS
     except AttributeError:
         skipped_local = []
     skipped_fields = skipped_local if skipped_local else skipped_global
@@ -261,6 +325,15 @@ def _clean(data, field_list):
         data.pop(field, None)
 
 
+def erase_special_metakeys(request):
+    new_meta = {}
+    for k, v in request.meta.items():
+        if not k.startswith('_'):
+            new_meta[k] = v
+    new_req = request.replace(meta=new_meta)
+    return new_req
+
+
 def write_test(path, test_name, url):
     command = 'scrapy {}'.format(' '.join(sys.argv))
     test_path = os.path.join(path, 'test_fixtures.py')
@@ -285,7 +358,7 @@ class TestMaster(unittest.TestCase):
         self.maxDiff = None
         for f in files:
             file_path = os.path.join(os.path.dirname(__file__), f)
-            print("Testing fixture '%s'" % (f))
+            print("Testing fixture '%s' in location: %s" % (f, file_path))
             test = generate_test(os.path.abspath(file_path))
             test(self)
 
@@ -304,8 +377,8 @@ if __name__ == '__main__':
     if not os.path.exists(config_file):
         config_src = os.path.dirname(__file__) + '/config_doc.py'
         shutil.copyfile(config_src, config_file)
-        
-        
+
+
 def binary_check(fx_obj, cb_obj, encoding):
     if isinstance(cb_obj, (dict, Item)):
         fx_obj = {
@@ -385,7 +458,7 @@ def prepare_callback_replay(fixture_path, encoding="utf-8"):
     crawler = Crawler(spider_cls, settings)
     spider_args_in = data.get('spider_args', data.get('spider_args_in', {}))
     spider = spider_cls.from_crawler(crawler)
-    for k,v in spider_args_in.items():
+    for k, v in spider_args_in.items():
         setattr(spider, k, v)
     crawler.spider = spider
 
@@ -428,7 +501,7 @@ def generate_test(fixture_path, encoding='utf-8'):
             if k not in ('crawler', 'settings', 'start_urls')
         }
         self.assertEqual(spider_args_in, result_attr_in,
-                         'Input arguments not equal!')
+                         'Input arguments not equal!\nFixture path: %s' % fixture_path)
 
         for mw in middlewares:
             if hasattr(mw, 'process_spider_input'):
@@ -448,21 +521,41 @@ def generate_test(fixture_path, encoding='utf-8'):
                 raise AssertionError(
                     "The fixture's data length doesn't match with "
                     "the current callback's output length. "
-                    "Expected %s elements, found %s" % (
-                        len(fx_result), index + 1 + len(list(result)))
+                    "Expected %s elements, found %s.\nFixture path: %s" % (
+                        len(fx_result), index + 1 + len(list(result)),
+                        fixture_path)
                 )
 
-            cb_settings = get_cb_settings(fixture_path)
+            test_dir = '/'.join(fixture_path.split('/')[:-1])
+            cb_settings = get_cb_settings(test_dir)
 
             cb_obj = parse_object(cb_obj, spider, cb_settings)
 
             fx_obj = fx_item['data']
             if fx_item['type'] == 'request':
-                clean_request(fx_obj, settings, cb_settings)
-                clean_request(cb_obj, settings, cb_settings)
+                fx_obj = clean_request(fx_obj, settings, cb_settings)
+                cb_obj = clean_request(cb_obj, settings, cb_settings)
+                result_to_validate = {'type': 'request', 'data': cb_obj}
+                try:
+                    validate_results(test_dir, settings, [result_to_validate], request.url)
+                except _InvalidOutput as e:
+                    six.raise_from(
+                        _InvalidOutput(
+                            "Callback output #{} is invalid "
+                            "Problem: {}.\nFixture path: {}".format(index, e, fixture_path)),
+                        None)
             else:
                 clean_item(fx_obj, settings, cb_settings)
                 clean_item(cb_obj, settings, cb_settings)
+                result_to_validate = {'type': 'item', 'data': cb_obj}
+                try:
+                    validate_results(fixture_path, settings, [result_to_validate], request.url)
+                except _InvalidOutput as e:
+                    six.raise_from(
+                        _InvalidOutput(
+                            "Callback output #{} is invalid "
+                            "Problem: {}.\nFixture path: {}".format(index, e, fixture_path)),
+                        None)
 
             if fx_version == 2 and six.PY3:
                 fx_obj = binary_check(fx_obj, cb_obj, encoding)
@@ -473,7 +566,7 @@ def generate_test(fixture_path, encoding='utf-8'):
                 six.raise_from(
                     AssertionError(
                         "Callback output #{} doesn't match recorded "
-                        "output:{}".format(index, e)),
+                        "output: {}.\nFixture path: {}".format(index, e, fixture_path)),
                     None)
 
         # Spider attributes get updated after the yield
@@ -483,5 +576,5 @@ def generate_test(fixture_path, encoding='utf-8'):
         }
 
         self.assertEqual(data['spider_args_out'], result_attr_out,
-                         'Output arguments not equal!')
-    return test
\ No newline at end of file
+                         'Output arguments not equal!\nFixture path: %s' % fixture_path)
+    return test
